// *****************************************************************************
// Source code for the FontPopUp Dialog in DG Test Add-On
// API Development Kit 20; Mac/Win
//
// Namespaces:			Contact person:
//		-None-						MoT
//
// [SG compatible] - Yes
// *****************************************************************************


// ---------------------------------- Includes ---------------------------------

#include "FontPopupDialog.hpp"

#include	"APIEnvir.h"
#include	"ACAPinc.h"					// also includes APIdefs.h
#include	"APICommon.h"
#include	"DGRichEdit.hpp"
#include	"DGUtility.hpp"
#include	"DGItemProperty.hpp"
#include	"DGTEFontData.hpp"


// --- Class definition: FontPopUpDialog -------------------------------------------------

FontPopupDialog::FontPopupDialog () :
	DG::ModalDialog (ACAPI_GetOwnResModule(), DGTEST_FONTPOPUPDIALOG_RESID, ACAPI_GetOwnResModule()),
	UC::FontPopUpObserver (),
	fontPopUp		(GetReference (), FontPopUpID),
	richEdit		(GetReference (), RichEditID),
	boldCheck		(GetReference (), BoldCheckID),
	italicCheck		(GetReference (), ItalicCheckID),
	underlineCheck	(GetReference (), UnderlineCheckID),
	closeButton		(GetReference (), CloseButtonID)
{
	fontPopUp.Attach (*this);
	boldCheck.Attach (*this);
	italicCheck.Attach (*this);
	underlineCheck.Attach (*this);
	closeButton.Attach (*this);
	
	fontPopUp.SelectItem (TE::FontFamily ("Arial"));
	richEdit.SetText ("Hello World");
	UpdateFont ();
}

FontPopupDialog::~FontPopupDialog ()
{
}


void FontPopupDialog::CheckItemChanged (const DG::CheckItemChangeEvent& /*ev*/)
{
	UpdateFont ();
}

void FontPopupDialog::FontPopUpChanged (const UC::FontPopUpChangeEvent& /*ev*/)
{
	UpdateFont ();
}


void FontPopupDialog::ButtonClicked (const DG::ButtonClickEvent& ev)
{
	if (ev.GetSource () == &closeButton)
		PostCloseRequest (Cancel);
}


void FontPopupDialog::UpdateFont ()
{
	DGTEFontData data;
	if (!richEdit.GetFont (&data)) {
		return;
	};
	data.font.Set (fontPopUp.GetSelectedItem ());
	Int32 fontStyle = 
		(boldCheck.IsChecked () ? TE::IFont::FontStyle::bold : 0) |
		(italicCheck.IsChecked () ? TE::IFont::FontStyle::italic : 0) |
		(underlineCheck.IsChecked () ? TE::IFont::FontStyle::underline : 0);

	data.font.SetStyle (fontStyle);
	richEdit.SetFont (&data);
}

